    module run_star_extras
  
      use star_lib
      use star_def
      use const_def
      use chem_def !to be able to call isotopes in a network independent fashion                                                                              
                   !only assuming the isotopes used ARE in the network                                                                                        
      implicit none                                                                                                                                           
      integer time0, time1, clock_rate !variables defined for the timing               

      
      ! these routines are called by the standard run_star check_model
      contains


!------------------------------------------------------------------------
        ! !CUSTOMIZED MESH CONTROL
        ! subroutine how_many_other_mesh_fcns(id, n)
        !   integer, intent(in) :: id
        !   integer, intent(out) :: n
        !   n = 2
        ! end subroutine how_many_other_mesh_fcns

        ! subroutine other_mesh_fcn_data( &
        !      id, nfcns, names, gval_is_xa_function, vals1, ierr)
        !   use crlibm_lib                                                                                                                    
        !   integer, intent(in) :: id
        !   integer, intent(in) :: nfcns
        !   character (len=*) :: names(:)
        !   logical, intent(out) :: gval_is_xa_function(:) ! (nfcns)                                                                                            $
        !   real(dp), pointer :: vals1(:) ! =(nz $\times$ nfcns)                                                                                                $
        !   integer, intent(out) :: ierr
        !   integer :: nz, k
        !   real(dp), pointer :: vals(:,:)
        !   real(dp), parameter :: weight = 10d0
        !   type (star_info), pointer :: s
        !   ierr = 0
        !   call star_ptr(id, s, ierr)
        !   if (ierr /= 0) return
        !   nz = s% nz
        !   names(1) = 'gradT'
        !   names(2) = 'mass'
        !   gval_is_xa_function(1) = .false.
        !   gval_is_xa_function(2) = .false.
        !   vals(1:nz,1:nfcns) => vals1(1:nz*nfcns)
        !   do k=1,nz
        !      vals(k,1) = weight * log_cr(s% equ(s% i_dlnT_dm, k)+1) ! +1 means that if dlnT_dm(k)=0 = > vals = 0 => no mesh refinement
        !      ! the use of a log scale is to limit roundoff errors
        !      ! weight is almost arbitrary, as long as weight > 0
        !      vals(k,2) = 100*weight * min(s%m(1)/Msun,max(s%m(1)/Msun-0.5, s%m(k)/Msun))
        !   end do
        ! end subroutine other_mesh_fcn_data


!-------------------------------------------------------------------------------------------------------------
    subroutine  Dutch_tanh_intrp_wind(id, Lsurf, Msurf, Rsurf, Tsurf, w, ierr)                                                                  
         use crlibm_lib                                                                                                                    
         type (star_info), pointer :: s                                                                                                                      
         integer, intent(in) :: id                                                                                                                           
         real(dp), intent(in) :: Lsurf, Msurf, Rsurf, Tsurf ! surface values (cgs)                                                                           
         ! NOTE: surface is outermost cell. not necessarily at photosphere.                                                                                  
         ! NOTE: don't assume that vars are set at this point.                                                                                               
         ! so if you want values other than those given as args,                                                                                             
         ! you should use values from s% xh(:,:) and s% xa(:,:) only.                                                                                        
         ! rather than things like s% Teff or s% lnT(:) which have not been set yet.                                                                         
         real(dp), intent(out) :: w ! wind in units of Msun/year (value is >= 0)                                                                             
         integer, intent(out) :: ierr                                                                                                                        
         real(dp) ::  L1, M1, R1, T1, xsurf, etaWR, etaHOT, etaCOOL, Zsolar, Y, Z
         real(dp) :: log10w, w1, w2, T_high, T_low, alfa                                                                                            
         call get_star_ptr(id,s,ierr)                                                                                                                
         w = 0                                                                                                                                       
         ierr = 0                                                                                                                                    
         L1 = Lsurf                                                                                                                                  
         M1 = Msurf                                                                                                                                  
         R1 = Rsurf                                                                                                                                  
         T1 = Tsurf                                                                                                                                  
         Zsolar = 0.019         
         !I use the same factor everywhere                          
         etaHOT = s% x_ctrl(1)
         etaCOOL = s% x_ctrl(2)
         etaWR = s% x_ctrl(3) 
         !Assumes the use of approx21.net           
         xsurf = s%xa(s%net_iso(iprot), 1)+ s%xa(s%net_iso(ih1), 1) !H mass fraction of the outermost cell, species 1 being neutrons                                                         
         ! Assumes to NOT use approx* family of networks
         !xsurf = s%xa(s%net_iso(ih1), 1) + s%xa(s%net_iso(ih2), 1)
         Y = s% xa(s%net_iso(ihe3),1) + s% xa(s%net_iso(ihe4),1)                                                                 
         Z = 1 - Y -xsurf          
         T_high = 11000d0                                                                                                                              
         T_low = 10000d0     
        if (xsurf .GE. 0.4d0) then                                                                                                            
            if (T1 <= T_low) then      
                print *, 'using: de_Jager, eta=', etaCOOL
                call eval_de_Jager_wind(w)                                                                                                               
                w = w * etaCOOL
            else if (T1 >= T_high) then                            
                print *, 'using: Vink_tanh, eta=', etaHOT
                call eval_tanh_intrp_vink_wind(w)                                                                                                            
                w = w * etaHOT
            else               ! transition
               print *, 'interpolating de Jager and Vink_tanh, eta=', etaHOT
                call eval_de_Jager_wind(w1)                                                                                                                 
                call eval_tanh_intrp_vink_wind(w2)                                                                                                
                alfa = (T1 - T_low)/(T_high - T_low)                                                                                                     
                w = (1-alfa)*w1 + alfa*w2                                                                                                             
                w = w * etaHOT
             end if 
         else !means it's a WR star
            print *, 'using: N&L, eta=', etaWR
            w = 1d-11 * pow_cr(L1/Lsun,1.29d0) * pow_cr(Y,1.7d0) * sqrt(Z) 
            w = w * etaWR
         end if 

         contains
           
        subroutine eval_de_Jager_wind(w)
        ! de Jager, C., Nieuwenhuijzen, H., & van der Hucht, K. A. 1988, A&AS, 72, 259.                                             
         real(dp), intent(out) :: w
            real(dp) :: log10w
            include 'formats'
            log10w = 1.769d0*log10_cr(L1/Lsun) - 1.676d0*log10_cr(T1) - 8.158d0                                                                              
            w = exp10_cr(log10w)
         end subroutine eval_de_Jager_wind


     subroutine eval_tanh_intrp_vink_wind(w)
          real(dp), intent(inout) :: w
          real(dp) :: alfa, w1, w2, w_c, w_h, Teff_jump, logMdot, dT, vinf_div_vesc   
          w = 0
          
         if (T1 > 27500d0) then
             vinf_div_vesc = 2.6d0 ! this is the hot side galactic value                                                                                    
             vinf_div_vesc = vinf_div_vesc*pow_cr(Z/Zsolar,0.13d0) ! corrected for Z                                                                        
              logMdot = &
                  - 6.697d0 &
                  + 2.194d0*log10_cr(L1/Lsun/1d5) &
                  - 1.313d0*log10_cr(M1/Msun/30) &
                  - 1.226d0*log10_cr(vinf_div_vesc/2d0) &
                  + 0.933d0*log10_cr(T1/4d4) &
                  - 10.92d0*pow2(log10_cr(T1/4d4)) &
                  + 0.85d0*log10_cr(Z/Zsolar)
               w1 = exp10_cr(logMdot)
               w = w1
            else if (T1 < 22500d0) then
        vinf_div_vesc = 1.3d0 ! this is the cool side galactic value   
        vinf_div_vesc = vinf_div_vesc*pow_cr(Z/Zsolar,0.13d0) ! corrected for Z                                                
               logMdot = &
                  - 6.688d0 &
                  + 2.210d0*log10_cr(L1/Lsun/1d5) &
                  - 1.339d0*log10_cr(M1/Msun/30) &
                  - 1.601d0*log10_cr(vinf_div_vesc/2d0) &
                  + 1.07d0*log10_cr(T1/2d4) &
                  + 0.85d0*log10_cr(Z/Zsolar)
               w2 = exp10_cr(logMdot)
               w = w2
            else ! use Vink et al 2001, eqns 14 and 15 to set "jump" temperature 
               Teff_jump = 1d3*(61.2d0 + 2.59d0*(-13.636d0 + 0.889d0*log10_cr(Z/Zsolar)))                     
               dT = 100d0
              ! print *, 'Teff_jump[K]', Teff_jump 
!     first evaluate the mass loss rate one would have if T = 27500, all rest held constant
               vinf_div_vesc = 2.6d0 ! this is the hot side galactic value
               vinf_div_vesc = vinf_div_vesc*pow_cr(Z/Zsolar,0.13d0) ! corrected for Z                                                
               w_h =  &
                  - 6.697d0 &
                  + 2.194d0*log10_cr(L1/Lsun/1d5) &
                  - 1.313d0*log10_cr(M1/Msun/30) &
                  - 1.226d0*log10_cr(vinf_div_vesc/2d0) &
                  + 0.933d0*log10_cr(27500/4d4) &
                  - 10.92d0*pow2(log10_cr(27500/4d4)) &
                  + 0.85d0*log10_cr(Z/Zsolar)
               w_h = exp10_cr(w_h)
               !then evaluate the mass loss rate one would have if T= 22500, all rest held constant
               vinf_div_vesc = 1.3d0 ! this is the cool side galactic value                                                                                  
               vinf_div_vesc = vinf_div_vesc*pow_cr(Z/Zsolar,0.13d0) ! corrected for Z                                                                       
               w_c = &
                  - 6.688d0 &
                  + 2.210d0*log10_cr(L1/Lsun/1d5) &
                  - 1.339d0*log10_cr(M1/Msun/30) &
                  - 1.601d0*log10_cr(vinf_div_vesc/2d0) &
                  + 1.07d0*log10_cr(22500/2d4) &
                  + 0.85d0*log10_cr(Z/Zsolar)
               w_c = exp10_cr(w_c)
               !then interpolate between this two values
               w = ((w_h - w_c)/2)*tanh_cr((T1-Teff_jump)/(1d0*dT)) &
                   +(w_c+w_h)/2
            end if

           end subroutine eval_tanh_intrp_vink_wind


      end subroutine  Dutch_tanh_intrp_wind

        subroutine extras_controls(id, ierr)
         integer, intent(in) :: id
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         
         ! this is the place to set any procedure pointers you want to change
         ! e.g., other_wind, other_mixing, other_energy  (see star_data.inc)
         s% other_wind =>  Dutch_tanh_intrp_wind       
         !s% how_many_other_mesh_fcns => how_many_other_mesh_fcns                         
         !s% other_mesh_fcn_data => other_mesh_fcn_data                                   
       end subroutine extras_controls
       
       
       integer function extras_startup(id, restart, ierr)
         integer, intent(in) :: id
         logical, intent(in) :: restart
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         extras_startup = 0
         if (.not. restart) then
            call alloc_extra_info(s)
         else ! it is a restart
            call unpack_extra_info(s)
         end if
         call system_clock(time0,clock_rate)
      end function extras_startup
      

      ! returns either keep_going, retry, backup, or terminate.
      integer function extras_check_model(id, id_extra)
         integer, intent(in) :: id, id_extra
         integer :: ierr
         type (star_info), pointer :: s
         real(dp) :: Delta_err
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         extras_check_model = keep_going 
         Delta_err = 0d0

         !to stop at log_center_T >= 9
         !use the following in you inlist
         !  log_center_temp_limit = 9.4
         !  when_to_stop_rtol = 5d-9
         !  when_to_stop_atol = 1d-3
         
         !to stop at carbon depletion do something similar to below

         !to stop at the onset of core collapse use the following in the inlist
         !  fe_core_infall_limit = 1d8
         
         ! !to stop at Oxygen depletion
         !  if (s% xa(s%net_iso(io16),s%nz) .LT. 0.04d0) then
         !     if (s%xa(s%net_iso(isi28), s%nz) .GT. 0.01d0) then !O depletion: X_o16 < 0.04 in the center and si28 has been produced                           $
         !        Delta_err = 1d0* abs(s% xa(s%net_iso(io16),s%nz) - 0.04d0)/abs(s% xa(s%net_iso(io16),s%nz))
         !        if (Delta_err .GT. 1d-4) then
         !           extras_check_model = retry
         !           print *, '***********************'
         !           print *, 'Approaching O depletion'
         !           print *, 'Delta_err = ', Delta_err
         !           print *, '***********************'
         !        end if
         !     end if
         !  end if
        
         if (.false. .and. s% star_mass_h1 < 0.35d0) then
            ! stop when star hydrogen mass drops to specified level
            extras_check_model = terminate
            write(*, *) 'have reached desired hydrogen mass'
            return
         end if

         ! by default, indicate where (in the code) MESA terminated
         if (extras_check_model == terminate) s% termination_code = t_extras_check_model
      end function extras_check_model


      integer function how_many_extra_history_columns(id, id_extra)
         integer, intent(in) :: id, id_extra
         integer :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         how_many_extra_history_columns = 0
      end function how_many_extra_history_columns
      
      
      subroutine data_for_extra_history_columns(id, id_extra, n, names, vals, ierr)
         integer, intent(in) :: id, id_extra, n
         character (len=maxlen_history_column_name) :: names(n)
         real(dp) :: vals(n)
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         
         !note: do NOT add the extras names to history_columns.list
         ! the history_columns.list is only for the built-in log column options.
         ! it must not include the new column names you are adding here.
         

      end subroutine data_for_extra_history_columns

      
      integer function how_many_extra_profile_columns(id, id_extra)
         use star_def, only: star_info
         integer, intent(in) :: id, id_extra
         integer :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         how_many_extra_profile_columns = 0
      end function how_many_extra_profile_columns
      
      
      subroutine data_for_extra_profile_columns(id, id_extra, n, nz, names, vals, ierr)
         use star_def, only: star_info, maxlen_profile_column_name
         use const_def, only: dp
         integer, intent(in) :: id, id_extra, n, nz
         character (len=maxlen_profile_column_name) :: names(n)
         real(dp) :: vals(nz,n)
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         integer :: k
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         
         !note: do NOT add the extra names to profile_columns.list
         ! the profile_columns.list is only for the built-in profile column options.
         ! it must not include the new column names you are adding here.

         ! here is an example for adding a profile column
         !if (n /= 1) stop 'data_for_extra_profile_columns'
         !names(1) = 'beta'
         !do k = 1, nz
         !   vals(k,1) = s% Pgas(k)/s% P(k)
         !end do
         
      end subroutine data_for_extra_profile_columns
      

      ! returns either keep_going or terminate.
      ! note: cannot request retry or backup; extras_check_model can do that.
      integer function extras_finish_step(id, id_extra)
        use crlibm_lib
        integer, intent(in) :: id, id_extra
        integer :: ierr, k
        type (star_info), pointer :: s
        real(dp) :: time, mdot_timescale, xsurf
    
        ierr = 0
        call star_ptr(id, s, ierr)
        if (ierr /= 0) return
        extras_finish_step = keep_going
        call store_extra_info(s)

        call system_clock(time1,clock_rate)
        time = dble(time1-time0)/ clock_rate / 60 ! elapsed time in minutes
        extras_finish_step = keep_going
        call store_extra_info(s)

         if (time .GT. 255) then !255 min = 4:55h
            call output(id, ierr) !to save a photo
            extras_finish_step = terminate
            print *, 'Terminated because maximum time reached'
         end if
         call get_star_ptr(id,s,ierr)
         
         s% lxtra1 = .false. !to check end hot phase
         if (s%xa(s%net_iso(ih1), s%nz) .LE. 1d-3) then !TAMS
            s% lxtra1 = .true.
            if ((s%lxtra1_old .eqv. .false.) .AND. (s% lxtra1 .eqv. .true.)) then
               call output(id, ierr) !to save a photo and model at the end of the hot phase of the evolution
            end if
         end if

         s% lxtra2 = .false. !to check end hot phase
         if (s% Teff .LE. 15000) then !End of hot phase of the evolution
            s% lxtra2 = .true.
            if ((s%lxtra2_old .eqv. .false.) .AND. (s% lxtra2 .eqv. .true.)) then
               call output(id, ierr) !to save a photo and model at the end of the hot phase of the evolution
            end if
         end if

         s% lxtra3 = .false. !to check beginning of WR phase
         !xsurf = s%xa(s%net_iso(ih1), 1) + s%xa(s%net_iso(ih2), 1) !for non-approx21 nets
         xsurf = s%xa(s%net_iso(iprot), 1)+ s%xa(s%net_iso(ih1), 1) 
         if (xsurf .LT. 0.4) then 
            s% lxtra3 = .true.
            if ((s%lxtra3_old .eqv. .false.) .AND. (s% lxtra3 .eqv. .true.)) then !its a WR
               call output(id, ierr) 
            end if
         end if

         s% lxtra4 = .false.
         if (s% xa(s%net_iso(io16),s%nz) .LT. 0.04) then
            if (s%xa(s%net_iso(isi28), s%nz) .GT. 0.01) then !O depletion: X_o16 < 0.04 in the center and si28 has been produced
               extras_finish_step = keep_going !terminate !set this to keep_going to go pass O_depl
               s%lxtra4 = .true.
               if ((s%lxtra4_old .eqv. .false.) .AND. (s% lxtra4 .eqv. .true.)) then
                  call output(id, ierr) !to save a photo
               end if
            end if
         end if
         
         
         !control that the next timestep is smaller than the relevant timescales!                                                                               
       
         mdot_timescale = s% mstar / (1d0 * abs(s% mstar_dot)) ! in sec
         !print *, "proposed dt_next [yr]", s% dt_next / secyer
         !dt_next must be in seconds, but for numerical stability the min
         ! must be evaluated with stuff in years
         s% dt_next = min(s% dt_next / secyer , s% kh_timescale, &
                          mdot_timescale / secyer)
         s% dt_next = s% dt_next * secyer

         if (min(s% dt_next / secyer , s% kh_timescale, &
              mdot_timescale / secyer) & 
              == s% kh_timescale ) then
            print *, "dt_next limited by kh_timescale"
         else if (min(s% dt_next / secyer , s% kh_timescale, &
              mdot_timescale / secyer) &
              == mdot_timescale / secyer )  then
            print *, "dt_next limited by mdot_timescale"
         end if
                  
       contains
         
         subroutine output(id, ierr)
           ! use star_utils, only: get_name_for_restart_file
           use crlibm_lib
           interface
              subroutine save_restart_info(iounit, id, ierr)
                integer, intent(in) :: iounit
                integer, intent(in) :: id
                integer, intent(out) :: ierr
              end subroutine save_restart_info
           end interface
           integer, intent(in) :: id
           integer, intent(out) :: ierr     
           
           character (len=256) :: filename, num_str, fstring, modname
           type (star_info), pointer :: s
           integer :: num_digits
           
           call get_star_ptr(id, s, ierr)
           if (ierr /= 0) return
           num_str = 'photo_4h55min'          

           if ((s%lxtra1_old .eqv. .false.) .and. (s%lxtra1 .eqv. .true.)) then
              num_str = 'photo_TAMS'
              call star_write_model(id, 'TAMS.mod' , ierr)
              s% need_to_update_history_now = .true.
              s% need_to_save_profiles_now = .true.
              s% save_profiles_model_priority = 100 !for TAMS
           end if
           
           if ((s%lxtra2_old .eqv. .false.) .and. (s%lxtra2 .eqv. .true.)) then
              num_str = 'photo_end_hot_phase'
              call star_write_model(id, 'end_hot.mod' , ierr)
              s% need_to_update_history_now = .true.
              s% need_to_save_profiles_now = .true.
              s% save_profiles_model_priority = 101 !for end of the hot phase of evolution
           end if

           if ((s%lxtra3_old .eqv. .false.) .and. (s%lxtra3 .eqv. .true.)) then
              num_str = 'photo_WR_start'
              call star_write_model(id, 'start_WR.mod' , ierr)
              s% need_to_update_history_now = .true.
              s% need_to_save_profiles_now = .true.
              s% save_profiles_model_priority = 102 !for becoming a WR
           end if


           if ((s% xa(s%net_iso(io16),s%nz) .LT. 0.04) .AND. (s%xa(s%net_iso(isi28), s%nz) .GT. 0.01)) then !O depletion: X_o16 < 0.04 in the center and si28 has been produced
              num_str = 'photo_O_depl'
              call star_write_model(id, 'O_depl.mod' , ierr)
              s% need_to_update_history_now = .true.
              s% need_to_save_profiles_now = .true.
              s% save_profiles_model_priority = 103 !for O depletion
           end if
           
           filename = trim(s% photo_directory) // '/' // trim(num_str) 
           call output_to_file(filename, id, ierr) 
           
         end subroutine output
         
         
         subroutine output_to_file(filename, id, ierr)
         use utils_lib, only:alloc_iounit, free_iounit
         use write_model
         character (len=*) :: filename
         integer, intent(in) :: id
         integer, intent(out) :: ierr
         
         integer :: iounit, k
         type (star_info), pointer :: s
         character(len=256) :: iomsg

         include 'formats'

         call get_star_ptr(id, s, ierr)
         if (ierr /= 0) return

         iounit = alloc_iounit(ierr); if (ierr /= 0) return
         open(iounit, file=trim(filename), action='write', &
         status='replace', iostat=ierr, iomsg=iomsg, form='unformatted')
         if (ierr == 0) then
            s% most_recent_photo_name = trim(filename)
           ! call output_star_model(s, iounit)
            close(iounit)
         else
            write(*,*) trim(iomsg)
         endif
         call free_iounit(iounit)
         
      end subroutine output_to_file


!         if (extras_finish_step == terminate) s% termination_code = t_extras_finish_step
      end function extras_finish_step
      
      
      subroutine extras_after_evolve(id, id_extra, ierr)
         integer, intent(in) :: id, id_extra
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
      end subroutine extras_after_evolve
      
      
      ! routines for saving and restoring extra data so can do restarts
         
         ! put these defs at the top and delete from the following routines
         !integer, parameter :: extra_info_alloc = 1
         !integer, parameter :: extra_info_get = 2
         !integer, parameter :: extra_info_put = 3
      
      
      subroutine alloc_extra_info(s)
         integer, parameter :: extra_info_alloc = 1
         type (star_info), pointer :: s
         call move_extra_info(s,extra_info_alloc)
      end subroutine alloc_extra_info
      
      
      subroutine unpack_extra_info(s)
         integer, parameter :: extra_info_get = 2
         type (star_info), pointer :: s
         call move_extra_info(s,extra_info_get)
      end subroutine unpack_extra_info
      
      
      subroutine store_extra_info(s)
         integer, parameter :: extra_info_put = 3
         type (star_info), pointer :: s
         call move_extra_info(s,extra_info_put)
      end subroutine store_extra_info
      
      
      subroutine move_extra_info(s,op)
         integer, parameter :: extra_info_alloc = 1
         integer, parameter :: extra_info_get = 2
         integer, parameter :: extra_info_put = 3
         type (star_info), pointer :: s
         integer, intent(in) :: op
         
         integer :: i, j, num_ints, num_dbls, ierr
         
         i = 0
         ! call move_int or move_flg    
         num_ints = i
         
         i = 0
         ! call move_dbl       
         
         num_dbls = i
         
         if (op /= extra_info_alloc) return
         if (num_ints == 0 .and. num_dbls == 0) return
         
         ierr = 0
         call star_alloc_extras(s% id, num_ints, num_dbls, ierr)
         if (ierr /= 0) then
            write(*,*) 'failed in star_alloc_extras'
            write(*,*) 'alloc_extras num_ints', num_ints
            write(*,*) 'alloc_extras num_dbls', num_dbls
            stop 1
         end if
         
         contains
         
         subroutine move_dbl(dbl)
            real(dp) :: dbl
            i = i+1
            select case (op)
            case (extra_info_get)
               dbl = s% extra_work(i)
            case (extra_info_put)
               s% extra_work(i) = dbl
            end select
         end subroutine move_dbl
         
         subroutine move_int(int)
            integer :: int
            i = i+1
            select case (op)
            case (extra_info_get)
               int = s% extra_iwork(i)
            case (extra_info_put)
               s% extra_iwork(i) = int
            end select
         end subroutine move_int
         
         subroutine move_flg(flg)
            logical :: flg
            i = i+1
            select case (op)
            case (extra_info_get)
               flg = (s% extra_iwork(i) /= 0)
            case (extra_info_put)
               if (flg) then
                  s% extra_iwork(i) = 1
               else
                  s% extra_iwork(i) = 0
               end if
            end select
         end subroutine move_flg
      
      end subroutine move_extra_info

      end module run_star_extras
      
      
